package org.ten60.netkernel.mod.pds;

import java.io.File;

import org.netkernel.layer0.boot.BootUtils;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class DynamicH2RDBMSConfigAccessor extends StandardAccessorImpl
{
	private boolean mCommissioned=false;
	
	public DynamicH2RDBMSConfigAccessor()
	{	this.declareThreadSafe();		
		this.declareUnhandledExceptionsExpired(false);
	}

	@Override
	public void onSource(INKFRequestContext context) throws Exception
	{	Object src=null;
		String base;
		String dbpath=null;
		if(BootUtils.bootedFromJar(context.getKernelContext().getKernel().getConfiguration()))
		{	/*Running in Jar*/
			base=dbpath="mem:H2Database";
		}
		else
		{	base=context.source("netkernel:/config/netkernel.install.path", String.class);
			dbpath=base+"pds"+File.separator+"H2Database";
			dbpath=dbpath.replaceAll("%20", " ");
		}
		String src2=null;
		try
		{	src2=context.source(base+"etc/pdsH2Config.xml", String.class);
		}
		catch(Exception e)
		{	src2="<config>"+
				"<rdbms>"+
					"<jdbcDriver>org.h2.Driver</jdbcDriver>"+
					"<jdbcConnection></jdbcConnection>"+
					"<poolSize>8</poolSize>"+
				"</rdbms>"+
			"</config>";
		}
		src=src2.replaceFirst("<jdbcConnection>.*?</jdbcConnection>", "<jdbcConnection>jdbc:h2:"+dbpath+";CACHE_SIZE=1024</jdbcConnection>");
	
		//Attempt to Initialize Database - will only run once as hereafter the config will be cached.
		commissionH2(context, (String)src);
			
		context.createResponseFrom(src);
	}
	
	private synchronized void commissionH2(INKFRequestContext context, String src) throws NKFException
	{	if(!mCommissioned)
		{	mCommissioned=true;
			INKFRequest req=context.createRequest("active:sqlBatch");
			req.addArgument("operand", "res:/org/ten60/netkernel/mod/pds/sql/setupRDBMS.xml");
			req.addArgumentByValue("configuration", src);
			context.issueRequest(req);
		}			
	}
}
